/*
 * ----------------------------------------------------------------------------
 *
 *  Vancouver Gas Prices Plugin for Google Desktop Sidebar.
 *
 *  Author: Ilya Solnyshkin http://www.bizon.org/ilya
 *
 *  Info courtesy of www.VancouverGasPrices.com
 *
 *  CHANGELOG:
 *    2006-10-19  : First Release
 *    2007-01-02  : Updated the gadget with a unique GUID
 * ----------------------------------------------------------------------------
 */


var allCityNames = new Array(
"All Areas",
"Aldergrove",
"Burnaby",
"Coquitlam",
"Delta",
"Fort Langley",
"Ladner",
"Langley",
"Maple Ridge",
"New Westminster",
"North Vancouver",
"Pitt Meadows",
"Port Coquitlam",
"Port Moody",
"Richmond",
"Surrey",
"Tsawwassen",
"Vancouver",
"West Vancouver",
"White Rock");

var stationLogos = new Array(
"canadian_tire.jpg",
"chevron.jpg",
"costco.jpg",
"domo.jpg",
"esso.jpg",
"husky.jpg",
"mohawk.jpg",
"petro_canada.jpg",
"shell.jpg",
"super_save.jpg",
"superstore.jpg",
"tempo.jpg",
"turbo.jpg");

// Setup

var itemImageHeight = 16;  // the image's height in pixels
var pricesArray;
var itemsArray;
var imageArray;
 
plugin.SetFlags(gddPluginFlagNone, gddContentFlagHaveDetails + gddContentItemFlagNoRemove);
plugin.onShowOptionsDlg = ShowOptionsDlg;
SetDefaultOptions();
utils.setInterval(OnTimer, 1000 * 60 * 30);  // check once every 30 minutes
OnTimer();

function SetDefaultOptions()
{
    if (options.GetValue("City") == "")
    {
        SetCity("Richmond");
      }
}

function GetCity()
{
    return options.GetValue("City");
}

function SetCity(name)
{
    options.PutValue("City", name);
}

function OnViewOpen()
{
     //Load default options
    SetDefaultOptions();
    OnTimer();
}

function ShowOptionsDlg(wnd)
{
    wnd.AddControl(gddWndCtrlClassLabel, 0, "", strOptionsDlgLabel1Text, 10, 10, 280, 30);
    var list = wnd.AddControl(gddWndCtrlClassList, gddWndCtrlTypeListDrop, "city_list", allCityNames, 10, 35, 280, 2000);
    list.value = GetCity();
    wnd.onClose = OptionsDlgClosed;
}

function OptionsDlgClosed(wnd, code)
{
    if (code == gddIdOK)
    {
        var list = wnd.GetControl("city_list");
        SetCity(list.value);
        contentArea.RemoveAllContentItems();
        OnTimer();
      }
}

function OnTimer()
{
    var item = new VanGasPrices(GetCity());
    item = null;
}

function VanGasPrices(name)
{
    //var url = "http://www.vancouvergasprices.com/index.aspx?s=Y&fuel=A&area=" + name + "&tme_limit=12";
    var url = "http://www.vancouvergasprices.com/";
    if (GetCity() == allCityNames[0])
    {
       url = url + "index.aspx";
    }
    else
    {
       name = name.replace(" ", "_");
       url = "http://www.vancouvergasprices.com/" + name + "/index.aspx";
    }
    onStart();
    onGetData(url);
}

function onGetData(url)
{
	var http = new XMLHttpRequest();
      http.onreadystatechange = OnData;
      http.open("GET", url, true);
      http.setRequestHeader('Pragma', "no-cache");
      http.send();
      function OnData()
      {
        if (http.readyState == 4)
        { // complete?
    
            // If we cannot connect to the Google Video Server
            // Raise error
            if (http.status != 200)
            {
                return;
            }
              // parse xml
            parseRawHTML(http.responseText);
      		delete http;
      		http = null;
    		onFinish();
        }
      }
}

function GasPriceObject(price, address, station, area, hour, minutes, ampm, datetime)
{
    this.price = price;
    this.address = address;
    this.station = station;
    this.area = area;
    this.hour = hour;
    this.minutes = minutes;  
    this.ampm = ampm;
    this.datetime = datetime;
}


function onStart()
{
    contentArea.RemoveAllContentItems ();
    pricesArray = new Array();
    itemsArray = new Array();
	imageArray = new Array();

    pricesArray.length = 0;
    itemsArray.length = 0;
	imageArray.length = 0;
}


function addNewItem(price, address, station, area, hour, minutes, ampm, datetime)
{
    var item = new ContentItem();
    item.heading = price + " - " + area;
    item.snippet = price + " - " + station + " - " + address;
    item.source = address.replace("&", "&&");

	station = station.toLowerCase();
	station = station.replace(" ", "_");

	var logo = null;
    for (var i = 0; i < stationLogos.length; ++i)
    {
        if (0 == stationLogos[i].indexOf(station))
        {
			logo = graphics.loadImage(stationLogos[i]);
            break;
        }
    }
	imageArray.push(logo);
	if(null != logo)
    {
		item.image = logo;
	}
	item.flags = gddContentItemFlagNoRemove;
    item.onDrawItem = DrawItem;         // set handler for drawing the item
    item.onGetHeight = GetItemHeight;   // set handler for calculating the item's height
    itemsArray.push(item);
    var newGasPriceObj = new GasPriceObject(price, address, station, area, hour, minutes, ampm, datetime);
    pricesArray.push(newGasPriceObj); // append
}

function onFinish()
{
    if (itemsArray.length <= 0)
        return;
        
    for (var i = itemsArray.length - 1; i >=0; i--)
        contentArea.addContentItem(itemsArray[i], gddItemDisplayInSidebar);
}

function parseRawHTML(htmlPage)
{
    if (htmlPage == null) return;
    
	var TableBeginPos = htmlPage.indexOf("class=\"PriceTable\"");
    if (TableBeginPos > 0 )
    {
		TableStr = SubMyStr(htmlPage, "class=\"PriceTable\"", "</table>");
		if (null == TableStr)
        	return;
   
        var FirstRow = TableStr.indexOf("<tr class='PTableRow'>");
        if (-1 == FirstRow)
                return;
        
		TableStr = TableStr.substr(FirstRow, TableStr.length - FirstRow);
		var pid = "", price = "", station = "", area = "", datetime = "", location = "";

        matches = MatchMyStr(TableStr, "<tr class='PTableRow'>", "</tr>");
        for (i = 0; i < matches.length; i+=2)
        {
            
                pid = "";
                pid = SubMyStr(matches[i], "<td class=\"PColLow\" rowspan=\"2\"><a href='update_form.aspx\?pid=", "'>");
                
                price = "";
                price = SubMyStr(matches[i], "'>", "</a>");

                station = "";
                station = SubMyStr(matches[i], "<td align=\"left\" class=\"PStation\">", "</td>");

                area = "";
                area = SubMyStr(matches[i], "<td class=\"PList\" rowspan=\"2\" width=\"75\">", "</td>");

                datetime = "";
                datetime = SubMyStr(matches[i], "<td class=\"PList\" rowspan=\"2\">", "</td>");
                datetime = datetime.replace("<br>", ",");

                if (i+1 < matches.length)
                {                
                    location = "";
                    location = SubMyStr(matches[i+1], "<td class=\"PAddress\" colspan=\"2\">", "</td>");
                    addNewItem(price, location, station, area, "", "", "", datetime);
                }
         }
	}
}

function TrimString(sInString)
{
    sInString = sInString.replace( /^\s+/g, "" );// strip leading
    return sInString.replace( /\s+$/g, "" );// strip trailing
}

function SubMyStr(text, pattern1, pattern2)
{
    var subText = null;
    i1 = text.indexOf(pattern1);
    if (i1 != -1)
    {
        i1 += pattern1.length;
        if (pattern2 == null)
        {
            i2 = text.length;
        }
        else
        {
            i2 = text.indexOf(pattern2, i1);
        }
        if(i2 != -1)
        {
            subText = text.substr(i1, i2 - i1);
            subText = TrimString(subText);
        }
    }
    return subText;
}

function MatchMyStr(text, pattern1, pattern2)
{
    var matches = new Array();
    
    var subText = null;
    var i1 = 0, i2 = 0;
        
    while (-1 != i1)
    {
        i1 = text.indexOf(pattern1, i1);
        if (i1 != -1)
        {
            i1 += pattern1.length;
            i2 = text.indexOf(pattern2, i1);
            if(i2 != -1)
            {
                subText = text.substr(i1, i2 - i1);
                subText = TrimString(subText);
                matches.push(subText);
                i1 = i2 + pattern2.length;
            }
            else
            {
                break;
            }
        }
    }
    return matches;
}

// Called to find out the height of each content item
function GetItemHeight(item, target, graphics, width)
{
  return itemImageHeight + 2;
}

// Called to draw each content item
function DrawItem(item, target, graphics, x, y, width, height)
{
    var index = -1;
    for (var i = 0; i < itemsArray.length; ++i)
    {
        if (item == itemsArray[i])
        {
            index = i;//(itemsArray.length - 1) - i;
            break;
        }
    }

    if (index == -1)
        return;
        
	if(imageArray[index] != null)
    {
		graphics.DrawImage(x, y, 16, itemImageHeight, imageArray[index], 80);
	}
    x += 20;
    graphics.DrawText(x, y, width, height, pricesArray[index].price, gddColorNormalText, gddTextFlagSingleLine, gddFontExtraInfo);
    txtHeight = 16;
    x += graphics.GetTextWidth("  " + pricesArray[index].price, 0, gddFontNormal);
    graphics.DrawText(x, y, width, height, pricesArray[index].address, gddColorExtraInfo, gddTextFlagSingleLine, gddFontExtraInfo);
    
    if ( x + (graphics.GetTextWidth("  " + pricesArray[index].address, 0, gddFontNormal)) + (graphics.GetTextWidth("  " + pricesArray[index].datetime, 0, gddFontNormal)) <= width)
    {
        graphics.DrawText(0, y, width, height, pricesArray[index].datetime, gddColorExtraInfo, gddTextFlagRight, gddFontExtraInfo);
    }
}


function OnDetailsView(item)
{
    var obj = new Object();
    obj.title = item.heading;
    return obj;
}

function DebugOutput(str)
{
    var _debug_ = true;
    if ( _debug_ == true )    
    {
        alert(str);
    }

}

